import createDecorator from '@loafer/core/util/decorators/createDecorator';
import InjectableDefinition from '@loafer/context/definition/InjectableDefinition';
import { getInjectableDefinition } from '@loafer/context/util/metadata';

import GetAppContext from '@loafer/context';
import { DEFAULT_QUALIFIER, InjectableSterotype } from '@loafer/pouches/constants';
import DefaultPouchFactory from '@loafer/pouches/factory/implement/DefaultPouchFactory';
import PouchDefinitionRegistry from '@loafer/pouches/factory/registry/PouchDefinitionRegistry';

const Service = (qualifier: string | symbol = DEFAULT_QUALIFIER ) => createDecorator('Service', {
  classDecorator: <TFunction extends Function>(target: TFunction): TFunction | void => {
    let injectableDefinition: InjectableDefinition = getInjectableDefinition(target.prototype, false);
    if (undefined !== injectableDefinition) {
      throw new Error('Cannot apply @Injectable or @Injectable stereotype decorator multiple times.');
    }

    injectableDefinition = getInjectableDefinition(target, true);
    injectableDefinition.Stereotype = InjectableSterotype.SERVICE;
    injectableDefinition.Qualifier = qualifier;

    let pouchFactory: DefaultPouchFactory = GetAppContext().getPouchFactory();
    let pouchDefinitionRegistry = <PouchDefinitionRegistry>pouchFactory;
    pouchDefinitionRegistry.registerPouchDefinition(injectableDefinition);

    return target;
  },

});


export default Service;
