import PouchDefinition from '@loafer/pouches/factory/config/PouchDefinition';
import { InjectableSterotype, PouchScope } from '@loafer/pouches/constants';

class InjectableDefinition implements PouchDefinition {
  protected _type: Object;
  protected _stereotype: InjectableSterotype;
  protected _scope: PouchScope = PouchScope.SINGLETON;
  protected _qualifier: string | symbol = undefined;
  protected _postConstruct: Set<string> = undefined;
  protected _preDestroy: Set<string> = undefined;

  public constructor(type: Object) {
    this._type = type;
    this._stereotype = InjectableSterotype.INJECTABLE;
  }

  public get Type(): Object {
      return this._type;
  }

  public get Stereotype(): InjectableSterotype {
      return this._stereotype;
  }

  public set Stereotype(stereotype: InjectableSterotype) {
    this._stereotype = stereotype;
  }

  public get Scope(): PouchScope {
      return this._scope;
  }

  public set Scope(scope: PouchScope) {
    this._scope = scope;
  }

  public get Qualifier(): string | symbol {
      return this._qualifier;
  }

  public set Qualifier(qualifier: string | symbol) {
    this._qualifier = qualifier;
  }

  public get PostConstruct(): Set<string> {
      return this._postConstruct;
  }

  public addPostConstruct(postConstruct: string): void {
    if (undefined === this._postConstruct) {
      this._postConstruct = new Set();
    }
    this._postConstruct.add(postConstruct);
  }

  public get PreDestroy(): Set<string> {
      return this._preDestroy;
  }

  public addPreDestroy(preDestroy: string): void {
    if (undefined === this._preDestroy) {
      this._preDestroy = new Set();
    }
    this._preDestroy.add(preDestroy);
  }

  public isSingleton(): boolean {
    return this._scope === PouchScope.SINGLETON;
  }
  public isTransient(): boolean {
    return this._scope === PouchScope.TRANSIENT;
  }
}

export default InjectableDefinition;
