export interface DecoratorFactory {
  classDecorator?: ClassDecorator;
  propertyDecorator?: PropertyDecorator;
  parameterDecorator?: ParameterDecorator;
  methodDecorator?: MethodDecorator;
}

const createDecorator = (name: string, decoratorFactory: DecoratorFactory) => {
  return (...args: any[]) => {
    let params = [];
    for (let i = 0; i < args.length; i++) {
      if (args[i]) {
        params.push(args[i]);
      }
    }

    switch(params.length) {
    case 1:
      if (undefined !== decoratorFactory.classDecorator) {
        return decoratorFactory.classDecorator.apply(this, args);
      }
      throw new Error(`Cannot apply @${name} decorator on Class.`);
    case 2:
    case 1:
      if (undefined !== decoratorFactory.propertyDecorator) {
        return decoratorFactory.propertyDecorator.apply(this, args);
      }
      throw new Error(`Cannot apply @${name} decorator on Property.`);
    case 3:
      if(typeof args[2] === 'number') {
        if (undefined !== decoratorFactory.parameterDecorator) {
          return decoratorFactory.parameterDecorator.apply(this, args);
        }
        throw new Error(`Cannot apply @${name} decorator on Parameter.`);
      }
      if (undefined !== decoratorFactory.methodDecorator) {
        return decoratorFactory.methodDecorator.apply(this, args);
      }
      throw new Error(`Cannot apply @${name} decorator on Method.`);
    default:
      throw new Error(`@${name} decorators are not valid here!`);
    }
  };
};

export default createDecorator;
