export type MetadataDefinable<T> = {new(target: Function): T};

const getClassMetadata = <DefinitionType>(target: Object,
                                          metadataKey: any,
                                          definitionType: MetadataDefinable<DefinitionType>,
                                          isCreate: boolean = false): DefinitionType => {

  let definition: DefinitionType;
  if (Reflect.hasOwnMetadata(metadataKey, target) !== true) {
      if (!isCreate) {
          return undefined;
      }
      definition = Object.create(definitionType.prototype);
      definition.constructor.call(definition, target);

      Reflect.defineMetadata(metadataKey, definition, target);
  } else {
    definition = Reflect.getMetadata(metadataKey, target);
  }
  return definition;
};

export default getClassMetadata;
