import * as METADATA from '@loafer/pouches/constants';
import { DEFAULT_QUALIFIER, InjectSterotype } from '@loafer/pouches/constants';
import InjectConfig from '@loafer/pouches/decorator/InjectConfig';
import * as CoreConstants from '@loafer/core/constants';

export interface InjectItem {
  stereotype: InjectSterotype;
  decoratorType: CoreConstants.DecoratorType;
  propertyConfig?: InjectConfig;
  parameterConfigMap?: Map<number, InjectConfig>;
}

class InjectDefinition {
  protected target: Object;
  protected injectMap: Map<string | symbol, InjectItem>;

  public constructor(target: Object) {
    this.target = target;
    this.injectMap = new Map();
  }

  public get injectors(): Map<string | symbol, InjectItem> {
    return this.injectMap;
  }

  /**
   * addInject
   */
  public addInject(injectConfig: InjectConfig, propertyKey: string | symbol, parameterIndex?: number): void {
    if (undefined === injectConfig.qualifier) {
      injectConfig.qualifier = DEFAULT_QUALIFIER;
    }
    if (undefined === injectConfig.required) {
      injectConfig.required = false;
    }

    if(typeof parameterIndex === 'number') {
      this.addInjectParameter(injectConfig, propertyKey, parameterIndex);
    } else {
      this.addInjectProperty(injectConfig, propertyKey);
    }
  }

  /**
   * addValue
   */
  public addValue(value: string, propertyKey: string | symbol, parameterIndex?: number): void {
    if(typeof parameterIndex === 'number') {
      this.addValueParameter(value, propertyKey, parameterIndex);
    } else {
      this.addValueProperty(value, propertyKey);
    }
  }

  private addValueProperty(value: string, propertyKey: string | symbol): void {
    return;
  }
  private addValueParameter(value: string, propertyKey: string | symbol, parameterIndex: number): void {
    return;
  }



  private addInjectParameter(injectConfig: InjectConfig,
                             propertyKey: string | symbol, parameterIndex: number): void {
    let injectItem: InjectItem;
    let parameterTypes: any[] = Reflect.getMetadata(CoreConstants.DESIGN_PARAMTYPES, this.target, propertyKey);

    if (undefined === injectConfig.type) {
      injectConfig.type = parameterTypes[parameterIndex];
    }

    if (this.injectMap.has(propertyKey)) {
      injectItem = this.injectMap.get(propertyKey);
    } else {
      injectItem = {
        stereotype: InjectSterotype.INJECT,
        decoratorType: CoreConstants.DecoratorType.PARAMETER,
      };
      this.injectMap.set(propertyKey, injectItem);
    }

    if (null !== injectItem.parameterConfigMap) {
      if (injectItem.parameterConfigMap.has(parameterIndex)) {
        throw new Error(`Cannot apply @inject decorator on one parameter[${propertyKey}:${parameterIndex}] multiple times.`);
      }
    } else {
      injectItem.parameterConfigMap = new Map();
    }

    injectItem.parameterConfigMap.set(parameterIndex, injectConfig);

  }

  private addInjectProperty(injectConfig: InjectConfig, propertyKey: string | symbol): void {
    if (this.injectMap.has(propertyKey)) {
      throw new Error(`Cannot apply @inject decorator on one property[${propertyKey}] multiple times.`);
    }

    let propertyType: any = Reflect.getMetadata(CoreConstants.DESIGN_TYPE, this.target, propertyKey);

    if (undefined === injectConfig.type) {
      injectConfig.type = propertyType;
    }

    let injectItem: InjectItem = {
      stereotype: InjectSterotype.INJECT,
      decoratorType: CoreConstants.DecoratorType.PROPERTY,
      propertyConfig: injectConfig,
    };

    this.injectMap.set(propertyKey, injectItem);
  }

}

export default InjectDefinition;
