import {
  validateQualifier,
} from '@loafer/pouches/util/qualifier';

import PouchDefinition from '@loafer/pouches/factory/config/PouchDefinition';
import PouchDefinitionRegistry from '@loafer/pouches/factory/registry/PouchDefinitionRegistry';
import DefaultSingletonPouchRegistry from '@loafer/pouches/factory/implement/DefaultSingletonPouchRegistry';



abstract class AbstractPouchFactory extends DefaultSingletonPouchRegistry implements PouchDefinitionRegistry {
  protected pouchDefinitionMap: Map<Object, Map<string | symbol, PouchDefinition>>;
  public constructor() {
    super();
    this.pouchDefinitionMap = new Map();
  }

  public registerPouchDefinition(pouchDefinition: PouchDefinition): void {
    let type = pouchDefinition.Type;
    let qualifier = pouchDefinition.Qualifier;

    if (this.hasPouchDefinition(type, qualifier)) {
      throw new Error(`Pouch Definition[${type.constructor.name}:${qualifier}] is exist already`);
    }
    let map = this.pouchDefinitionMap.get(type);
    if (undefined === map) {
      map = new Map();
    }
    map.set(qualifier, pouchDefinition);
  }
  public removePouchDefinition(type: Object, qualifier?: string | symbol): void {
    if (!this.hasPouchDefinition(type, qualifier)) {
      throw new Error(`Pouch Definition[${type.constructor.name}:${qualifier}] is not exist`);
    }
    const _qualifier = validateQualifier(qualifier);
    this.pouchDefinitionMap.get(type).delete(_qualifier);
    if (0 === this.pouchDefinitionMap.get(type).size) {
      this.pouchDefinitionMap.delete(type);
    }
  }
  public getPouchDefinition(type: Object, qualifier?: string | symbol): PouchDefinition {
    if (!this.pouchDefinitionMap.has(type)) {
      return undefined;
    }
    const _qualifier = validateQualifier(qualifier);
    return this.pouchDefinitionMap.get(type).get(qualifier);
  }
  public hasPouchDefinition(type: Object, qualifier?: string | symbol): boolean {
    return undefined === this.getPouchDefinition(type, qualifier) ? false : true;
  }



}

export default AbstractPouchFactory;
