import {
  Construtorable,
} from '@loafer/core/constants';

import SingletonPouchRegistry from '@loafer/pouches/factory/config/SingletonPouchRegistry';

import {
  validateQualifier,
} from '@loafer/pouches/util/qualifier';

class DefaultSingletonPouchRegistry implements SingletonPouchRegistry {
  protected singletonInstanceMap: Map<Object, Map<string | symbol, any>>;

  protected constructor() {
  }

  public registerSingleton(pouch: any, qualifier?: string | symbol): void {
    let type = Object.getPrototypeOf(pouch);
    const _qualifier = validateQualifier(qualifier);

    if (this.hasSingleton(type, qualifier)) {
      throw new Error(`Pouch Definition[${type.constructor.name}:${qualifier}] is exist already`);
    }
    let map = this.singletonInstanceMap.get(type);
    if (undefined === map) {
      map = new Map();
    }
    map.set(qualifier, pouch);
  }
  public getSingleton<T>(type: Construtorable<T>, qualifier?: string | symbol): T {
    if (!this.singletonInstanceMap.has(type)) {
      return undefined;
    }
    const _qualifier = validateQualifier(qualifier);
    return this.singletonInstanceMap.get(type).get(qualifier);
  }
  public hasSingleton<T>(type: Construtorable<T>, qualifier?: string | symbol): boolean {
    return undefined === this.getSingleton(type, qualifier) ? false : true;
  }
}

export default DefaultSingletonPouchRegistry;
